import java.awt.Color;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.image.BufferedImage;
import java.io.IOException;
import java.util.ArrayList;
import java.util.LinkedList;
import java.util.List;
import java.util.Queue;

import javax.imageio.ImageIO;
import javax.swing.JPanel;


public class HeartbeatPanel extends JPanel{


	double scrollRate = 10;

	double currentX;

	Queue<Long> heartbeatQueue = new LinkedList<Long>();
	double bpm;

	List<Heartbeat> heartbeats = new ArrayList<Heartbeat>();

	List<Double> badBeats = new ArrayList<Double>();

	TypeYourHeartOut gamePanel;

	boolean alive = false;
	
	
	

	public HeartbeatPanel(TypeYourHeartOut gamePanel){
		this.gamePanel = gamePanel;

		setBackground(Color.BLACK);
		setPreferredSize(new Dimension(500, 100));

		heartbeats.add(new Heartbeat(500, 50));
		heartbeats.add(new Heartbeat(250, 50));
		
		
	}

	public void setAlive(boolean alive){
		this.alive = alive;
	}

	public void paintComponent(Graphics g){
		super.paintComponent(g);

		if(alive){
			currentX = getWidth()/2;

			g.setColor(Color.RED);
			g.setFont(new Font(Font.DIALOG, Font.PLAIN, 24));
			//g.drawString("BPM: " + bpm, 250, 25);

			for(double d : badBeats){
				g.fillOval((int)d, getHeight()/2, 10, 10);
			}

			for(Heartbeat heartbeat : heartbeats){
				heartbeat.draw(g);
			}

			g.setColor(Color.GREEN);
			g.drawLine((int)currentX, 0, (int)currentX, getHeight());
		}
		else{
			g.setColor(Color.RED);
			g.drawLine(0, getHeight()/2, getWidth(), getHeight()/2);
		}


	}



	public void step(){
		while(!heartbeatQueue.isEmpty() && heartbeatQueue.peek() < System.currentTimeMillis() - 5000){
			heartbeatQueue.poll();
		}

		bpm = (heartbeatQueue.size() / 5.0)*60;


		for(int i = badBeats.size()-1; i >= 0; i--){
			badBeats.set(i, badBeats.get(i) - scrollRate);
			if(badBeats.get(i) < -100){
				badBeats.remove(i);
			}
		}

		for(Heartbeat heartbeat : heartbeats){
			heartbeat.x -= scrollRate;
			if(heartbeat.x < -heartbeat.width){

				if(!heartbeat.good){
					gamePanel.missedBeat();
				}

				heartbeat.setGood(false);
				heartbeat.x = 500;
			}
		}




		repaint();
	}

	public boolean beat(){
		heartbeatQueue.add(System.currentTimeMillis());

		for(Heartbeat heartbeat : heartbeats){
			if(currentX > heartbeat.x && currentX < heartbeat.x + heartbeat.width){
				heartbeat.setGood(true);
				return true;
			}
		}

		badBeats.add(currentX);
		return false;
	}

	private class Heartbeat{

		double x;
		int width;
		boolean good;

		public Heartbeat(double x, int width){
			this.x = x;
			this.width = width;
		}

		public void setGood(boolean good){
			this.good = good;
		}

		public void draw(Graphics g){
			if(good){
				g.setColor(new Color(0, 255, 0, 255));
			}
			else{
				g.setColor(new Color(255, 0, 0, 255));
			}

			//g.fillRect((int) x, 0, width, getHeight());

			g.setFont(new Font(Font.DIALOG, Font.PLAIN, 72));

			g.drawString("\u2764", (int)x, 75);
		}
	}


}
